import math
import random 


"""
INSERT DESCRIPTION



"""

def getMedian(alist):
    """get median of list"""
    tmp = list(alist)
    tmp.sort()
    alen = len(tmp)
    if (alen % 2) == 1:
        return tmp[alen // 2]
    else:
        return (tmp[alen // 2] + tmp[(alen // 2) - 1]) / 2
    

def normalizeColumn(column):
    """normalize the values of a column using Modified Standard Score
    that is (each value - median) / (absolute standard deviation)"""
    median = getMedian(column)
    asd = sum([abs(x - median) for x in column]) / len(column)
    result = [(x - median) / asd for x in column]
    return result


class kClusterer:
    """ Implementation of kMeans Clustering
    This clusterer assumes that the first column of the data is a label
    not used in the clustering. The other columns contain numeric data"""
    
    def __init__(self, filename, k):
        """ k is the number of clusters to make"""
        file = open(filename)
        self.data = {}
        self.k = k
        self.counter = 0
        #
        # read data from file
        #
        lines = file.readlines()
        file.close()
        header = lines[0].split(',')
        self.cols = len(header)
        self.data = [[] for i in range(len(header))]
        for line in lines[1:]:
            cells = line.split(',')
            toggle = 0
            for cell in range(self.cols):
                if toggle == 0:
                   self.data[cell].append(cells[cell])
                   toggle = 1
                else:
                    self.data[cell].append(float(cells[cell]))
                    
        self.datasize = len(self.data[1])
        self.memberOf = [-1 for x in range(len(self.data[1]))]
        #
        # now normalize number columns
        #
        for i in range(1, self.cols):
                self.data[i] = normalizeColumn(self.data[i])

        ##
        ##   NOW THE K MEANS PART
        ##
        #  set up initial centroids
        random.seed()
        minMax = [(min(self.data[i]), max(self.data[i]))
                  for i in range(1, len(self.data))]
        #
        # create k random centroids
        #
        goodInitialPool = False
        #
        # this while loop makes sure each initial cluster has at least one member
        #
        while not goodInitialPool:
            self.centroids = [[random.uniform(minMax[i][0], minMax[i][1])
                               for i in range(len(minMax))] for x in range(k)]
            self.assignPointsToCluster()
            goodInitialPool = True
            for x in range(len(self.centroids)):
                if x not in self.memberOf:
                    goodInitialPool = False
            

    def updateCentroids(self):
        """Using the points in the clusters determines the centroid (mean point)
        of each cluster"""
        self.centroids = [[sum([hg.data[k][i]  for i in range(len(hg.data[0]))
                                if hg.memberOf[i] == centroid])/self.datasize
                           for k in range(1, len(self.data))]
                          for centroid in range(len(self.centroids))] 
            
        
    
    def assignPointToCluster(self, i):
        """ assign point to cluster based on distance from centroids"""
        min = 999999
        clusterNum = -1
        for centroid in range(self.k):
            dist = self.distance(i, centroid)
            if dist < min:
                min = dist
                clusterNum = centroid
        
        return clusterNum

    def assignPointsToCluster(self):
        """ assign each data point to a cluster"""
        self.memberOf = [self.assignPointToCluster(i)
                         for i in range(len(self.data[1]))]
        

        
    def distance(self, i, j):
        """ compute distance of point i from centroid j"""
        sumSquares = 0
        for k in range(1, self.cols):
            sumSquares += (self.data[k][i] - self.centroids[j][k-1])**2
        return math.sqrt(sumSquares)

    def kCluster(self):
        """the method that actually performs the clustering"""
        i = 0
        done = False
 
        while not done:
            previousCentroids = list(self.centroids)
            self.updateCentroids()
            self.assignPointsToCluster()
            i += 1
            #
            # we are done if either we have performed 100 iterations or
            # the centroids haven't changed
            #
            if i > 100 or previousCentroids == self.centroids:
                done = True

    def showMembers(self):
        """Display the results"""
        for centroid in range(len(self.centroids)):
             print ("\n\nClass %i\n========" % centroid)
             for name in [hg.data[0][i]  for i in range(len(hg.data[0]))
                          if hg.memberOf[i] == centroid]:
                 print (name)
        
    
                         

def xdistance(point1, point2):
    return math.sqrt(sum([(x[0] -x[1])**2 for x in zip(point1, point2)]))            



filename = '//Users/raz/Dropbox/guide/pg2dm-python/ch8/dogs2.csv'

hg = kClusterer(filename, 3)
hg.kCluster()
hg.showMembers()

print ("\n\n\n Cereal Data set\n\n")
hg = kClusterer('cer.csv', 4)
hg.kCluster()
hg.showMembers()


print ("\n\n\n Athletes\n\n")
hg = kClusterer('/Users/raz/Dropbox/guide/pg2dm-python/ch8/athlete.txt', 3)
hg.kCluster()
hg.showMembers()
#printDendrogram(cluster)



